@@ -21,6 +21,8 @@ import sys
 from spyderlib import __version__
 
 
+SUPPORTED_IPYTHON = '>=0.13;<1.0'
+
 #==============================================================================
 # Debug helpers
 #==============================================================================
@@ -102,6 +104,36 @@ SCIENTIFIC_STARTUP = get_module_source_path('spyderlib',
                                             'scientific_startup.py')
 
 
+#==============================================================================
+# Image path list
+#==============================================================================
+
+IMG_PATH = []
+def add_image_path(path):
+    if not osp.isdir(path):
+        return
+    global IMG_PATH
+    IMG_PATH.append(path)
+    for _root, dirs, _files in os.walk(path):
+        for dir in dirs:
+            IMG_PATH.append(osp.join(path, dir))
+
+add_image_path(get_module_data_path('spyderlib', relpath='images'))
+
+from spyderlib.otherplugins import PLUGIN_PATH
+if PLUGIN_PATH is not None:
+    add_image_path(osp.join(PLUGIN_PATH, 'images'))
+
+def get_image_path(name, default="not_found.png"):
+    """Return image absolute path"""
+    for img_path in IMG_PATH:
+        full_path = osp.join(img_path, name)
+        if osp.isfile(full_path):
+            return osp.abspath(full_path)
+    if default is not None:
+        return osp.abspath(osp.join(img_path, default))
+
+
 #==============================================================================
 # Translations
 #==============================================================================
