@@ -5,32 +5,26 @@
 # (see spyderlib/__init__.py for details)
 
 """
-Spyder GUI-related configuration management
-(for non-GUI configuration, see spyderlib/baseconfig.py)
+Spyder configuration options
 
-Important note regarding shortcuts:
-    For compatibility with QWERTZ keyboards, one must avoid using the following
-    shortcuts:
-        Ctrl + Alt + Q, W, F, G, Y, X, C, V, B, N
+Note: Leave this file free of Qt related imports, so that it can be used to
+quickly load a user config file
 """
 
 import os
 import sys
 import os.path as osp
 
-from spyderlib.qt.QtGui import QLabel, QIcon, QPixmap, QFont, QFontDatabase
-
 # Local import
-from spyderlib.userconfig import UserConfig, get_home_dir, NoDefault
-from spyderlib.baseconfig import (SUBFOLDER, CHECK_ALL, EXCLUDED_NAMES,
-                                  get_module_data_path, _)
+from spyderlib.userconfig import UserConfig, get_home_dir
+from spyderlib.baseconfig import SUBFOLDER, CHECK_ALL, EXCLUDED_NAMES, _
 from spyderlib.utils import iofuncs, codeanalysis
 
 
 SANS_SERIF = ['Sans Serif', 'DejaVu Sans', 'Bitstream Vera Sans',
-              'Bitstream Charter', 'Times', 'Lucida Grande', 'Calibri',
-              'MS Shell Dlg 2', 'Verdana', 'Geneva', 'Lucid', 'Arial',
-              'Helvetica', 'Avant Garde', 'sans-serif']
+              'Bitstream Charter', 'Lucida Grande', 'MS Shell Dlg 2',
+              'Calibri', 'Verdana', 'Geneva', 'Lucid', 'Arial',
+              'Helvetica', 'Avant Garde', 'Times', 'sans-serif']
 
 MONOSPACE = ['Monospace', 'DejaVu Sans Mono', 'Consolas', 'Monaco',
              'Bitstream Vera Sans Mono', 'Andale Mono', 'Liberation Mono',
@@ -55,12 +49,17 @@ EDIT_FILETYPES = (
     (_("C++ files"), ('.cc', '.cpp', '.cxx', '.h', '.hh', '.hpp', '.hxx')),
     (_("OpenCL files"), ('.cl', )),
     (_("Fortran files"), ('.f', '.for', '.f77', '.f90', '.f95', '.f2k')),
+    (_("IDL files"), ('.pro', )),
+    (_("MATLAB files"), ('.m', )),
     (_("Patch and diff files"), ('.patch', '.diff', '.rej')),
     (_("Batch files"), ('.bat', '.cmd')),
     (_("Text files"), ('.txt',)),
     (_("reStructured Text files"), ('.txt', '.rst')),
     (_("gettext files"), ('.po', '.pot')),
+    (_("NSIS files"), ('.nsi', '.nsh')),
     (_("Web page files"), ('.css', '.htm', '.html',)),
+    (_("XML files"), ('.xml',)),
+    (_("Enaml files"), ('.enaml',)),
     (_("Configuration files"), ('.properties', '.session', '.ini', '.inf',
                                 '.reg', '.cfg', '.desktop')),
                  )
@@ -117,9 +116,16 @@ EXCLUDE_PATTERNS = [r'\.pyc$|\.pyo$|\.orig$|\.hg|\.svn|\bbuild\b',
 NAME_FILTERS = ['*' + _ext for _ext in VALID_EXT + SHOW_EXT if _ext]+\
                ['README', 'INSTALL', 'LICENSE', 'CHANGELOG']
 
+# Port used to detect if there is a running instance and to communicate with
+# it to open external files
+OPEN_FILES_PORT = 21128
+
 DEFAULTS = [
             ('main',
              {
+              'single_instance': True,
+              'open_files_port': OPEN_FILES_PORT,
+              'tear_off_menus': False,
               'vertical_dockwidget_titlebars': False,
               'vertical_tabs': False,
               'animated_docks': True,
@@ -127,10 +133,22 @@ DEFAULTS = [
               'window/position': (10, 10),
               'window/is_maximized': False,
               'window/is_fullscreen': False,
+              'window/prefs_dialog_size': (745, 411),
               'lightwindow/size': (650, 400),
               'lightwindow/position': (30, 30),
               'lightwindow/is_maximized': False,
               'lightwindow/is_fullscreen': False,
+
+              # The following setting is currently not used but necessary from 
+              # a programmatical point of view (see spyder.py):
+              # (may become useful in the future if we add a button to change 
+              # settings within the "light mode")
+              'lightwindow/prefs_dialog_size': (745, 411),
+
+              'memory_usage/enable': True,
+              'memory_usage/timeout': 2000,
+              'cpu_usage/enable': False,
+              'cpu_usage/timeout': 2000,
               }),
             ('quick_layouts',
              {
@@ -214,10 +232,39 @@ DEFAULTS = [
               'umd/verbose': True,
               'umd/namelist': ['guidata', 'guiqwt'],
               'light_background': True,
-              'ipython_set_color': True,
               'merge_output_channels': os.name != 'nt',
               'colorize_sys_stderr': os.name != 'nt',
               }),
+            ('ipython_console',
+             {
+              'font/family': MONOSPACE,
+              'font/size': MEDIUM,
+              'font/italic': False,
+              'font/bold': False,
+              'show_banner': True,
+              'use_gui_completion': True,
+              'use_pager': True,
+              'show_calltips': False,
+              'ask_before_closing': True,
+              'object_inspector': True,
+              'buffer_size': 10000,
+              'pylab': True,
+              'pylab/autoload': True,
+              'pylab/backend': 0,
+              'pylab/inline/figure_format': 0,
+              'pylab/inline/resolution': 72,
+              'pylab/inline/width': 6,
+              'pylab/inline/height': 4,
+              'startup/run_lines': '',
+              'startup/use_run_file': False,
+              'startup/run_file': '',
+              'open_ipython_at_startup': False,
+              'greedy_completer': False,
+              'autocall': 0,
+              'symbolic_math': False,
+              'in_prompt': '',
+              'out_prompt': ''
+              }),
             ('variable_explorer',
              {
               'shortcut': "Ctrl+Shift+V",
@@ -261,6 +308,8 @@ DEFAULTS = [
               'calltips': True,
               'go_to_definition': True,
               'close_parentheses': True,
+              'close_quotes': False,
+              'add_colons': True,
               'auto_unindent': True,
               'indent_chars': '*    *',
               'tab_stop_width': 40,
@@ -306,6 +355,7 @@ DEFAULTS = [
               'rich_text/font/italic': False,
               'rich_text/font/bold': False,
               'wrap': True,
+              'math': True,
               'automatic_import': True,
               }),
             ('onlinehelp',
@@ -378,151 +428,8 @@ DEV = not __file__.startswith(sys.prefix)
 DEV = False
 CONF = UserConfig('spyder', defaults=DEFAULTS, load=(not DEV), version='2.4.0',
                   subfolder=SUBFOLDER, backup=True, raw_mode=True)
+
 # Removing old .spyder.ini location:
 old_location = osp.join(get_home_dir(), '.spyder.ini')
 if osp.isfile(old_location):
     os.remove(old_location)
-
-
-IMG_PATH = []
-def add_image_path(path):
-    if not osp.isdir(path):
-        return
-    global IMG_PATH
-    IMG_PATH.append(path)
-    for _root, dirs, _files in os.walk(path):
-        for dir in dirs:
-            IMG_PATH.append(osp.join(path, dir))
-
-add_image_path(get_module_data_path('spyderlib', relpath='images'))
-
-from spyderlib.otherplugins import PLUGIN_PATH
-if PLUGIN_PATH is not None:
-    add_image_path(osp.join(PLUGIN_PATH, 'images'))
-
-
-def get_image_path(name, default="not_found.png"):
-    """Return image absolute path"""
-    for img_path in IMG_PATH:
-        full_path = osp.join(img_path, name)
-        if osp.isfile(full_path):
-            return osp.abspath(full_path)
-    if default is not None:
-        return osp.abspath(osp.join(img_path, default))
-
-def get_icon( name, default=None ):
-    """Return image inside a QIcon object"""
-    if default is None:
-        return QIcon(get_image_path(name))
-    elif isinstance(default, QIcon):
-        icon_path = get_image_path(name, default=None)
-        return default if icon_path is None else QIcon(icon_path)
-    else:
-        return QIcon(get_image_path(name, default))
-
-def get_image_label( name, default="not_found.png" ):
-    """Return image inside a QLabel object"""
-    label = QLabel()
-    label.setPixmap(QPixmap(get_image_path(name, default)))
-    return label
-
-def font_is_installed(font):
-    """Check if font is installed"""
-    return [fam for fam in QFontDatabase().families() if unicode(fam)==font]
-    
-def get_family(families):
-    """Return the first installed font family in family list"""
-    if not isinstance(families, list):
-        families = [ families ]
-    for family in families:
-        if font_is_installed(family):
-            return family
-    else:
-        print "Warning: None of the following fonts is installed: %r" % families
-        return QFont().family()
-    
-FONT_CACHE = {}
-def get_font(section, option=None):
-    """Get console font properties depending on OS and user options"""
-    font = FONT_CACHE.get((section, option))
-    if font is None:
-        if option is None:
-            option = 'font'
-        else:
-            option += '/font'
-        families = CONF.get(section, option+"/family", None)
-        if families is None:
-            return QFont()
-        family = get_family(families)
-        weight = QFont.Normal
-        italic = CONF.get(section, option+'/italic', False)
-        if CONF.get(section, option+'/bold', False):
-            weight = QFont.Bold
-        size = CONF.get(section, option+'/size', 9)
-        font = QFont(family, size, weight)
-        font.setItalic(italic)
-        FONT_CACHE[(section, option)] = font
-    return font
-
-def set_font(font, section, option=None):
-    """Set font"""
-    if option is None:
-        option = 'font'
-    else:
-        option += '/font'
-    CONF.set(section, option+'/family', unicode(font.family()))
-    CONF.set(section, option+'/size', float(font.pointSize()))
-    CONF.set(section, option+'/italic', int(font.italic()))
-    CONF.set(section, option+'/bold', int(font.bold()))
-    FONT_CACHE[(section, option)] = font
-
-
-def get_shortcut(context, name, default=NoDefault):
-    """Get keyboard shortcut (key sequence string)"""
-    return CONF.get('shortcuts', '%s/%s' % (context, name), default=default)
-
-def set_shortcut(context, name, keystr):
-    """Set keyboard shortcut (key sequence string)"""
-    CONF.set('shortcuts', '%s/%s' % (context, name), keystr)
-    
-def iter_shortcuts():
-    """Iterate over keyboard shortcuts"""
-    for option in CONF.options('shortcuts'):
-        context, name = option.split("/", 1)
-        yield context, name, get_shortcut(context, name)
-        
-def reset_shortcuts():
-    """Reset keyboard shortcuts to default values"""
-    CONF.remove_section('shortcuts')
-
-
-from spyderlib.widgets.sourcecode.syntaxhighlighters import (
-                                COLOR_SCHEME_KEYS, COLOR_SCHEME_NAMES, COLORS)
-def get_color_scheme(name):
-    """Get syntax color scheme"""
-    color_scheme = {}
-    for key in COLOR_SCHEME_KEYS:
-        color_scheme[key] = CONF.get("color_schemes", "%s/%s" % (name, key))
-    return color_scheme
-
-def set_color_scheme(name, color_scheme, replace=True):
-    """Set syntax color scheme"""
-    section = "color_schemes"
-    names = CONF.get("color_schemes", "names", [])
-    for key in COLOR_SCHEME_KEYS:
-        option = "%s/%s" % (name, key)
-        value = CONF.get(section, option, default=None)
-        if value is None or replace or name not in names:
-            CONF.set(section, option, color_scheme[key])
-    names.append(unicode(name))
-    CONF.set(section, "names", sorted(list(set(names))))
-
-def set_default_color_scheme(name, replace=True):
-    """Reset color scheme to default values"""
-    assert name in COLOR_SCHEME_NAMES
-    set_color_scheme(name, COLORS[name], replace=replace)
-
-for _name in COLOR_SCHEME_NAMES:
-    set_default_color_scheme(_name, replace=False)
-CUSTOM_COLOR_SCHEME_NAME = "Custom"
-set_color_scheme(CUSTOM_COLOR_SCHEME_NAME, COLORS["Spyder"], replace=False)
