@@ -8,13 +8,17 @@
 
 import os
 import os.path as osp
+import sys
 
 from spyderlib.baseconfig import _
-from spyderlib.config import (get_icon, CONF, CUSTOM_COLOR_SCHEME_NAME,
-                              set_default_color_scheme, COLOR_SCHEME_NAMES)
-from spyderlib.utils.qthelpers import get_std_icon
+from spyderlib.config import CONF
+from spyderlib.guiconfig import (CUSTOM_COLOR_SCHEME_NAME,
+                                 set_default_color_scheme)
+from spyderlib.utils import programs
+from spyderlib.utils.qthelpers import get_icon, get_std_icon
 from spyderlib.userconfig import NoDefault
 from spyderlib.widgets.colors import ColorLayout
+from spyderlib.widgets.sourcecode import syntaxhighlighters as sh
 
 from spyderlib.qt.QtGui import (QWidget, QDialog, QListWidget, QListWidgetItem,
                                 QVBoxLayout, QStackedWidget, QListView,
@@ -29,6 +33,15 @@ from spyderlib.qt.compat import (to_qvariant, from_qvariant,
                                  getexistingdirectory, getopenfilename)
 
 
+class SizeMixin(object):
+    """Mixin to keep widget size accessible
+    even when C++ object has been deleted"""
+    def resizeEvent(self, event):
+        """Reimplement Qt method"""
+        QDialog.resizeEvent(self, event)
+        self.emit(SIGNAL("size_change(QSize)"), self.size())
+
+
 class ConfigPage(QWidget):
     """Configuration page base class"""
     def __init__(self, parent, apply_callback=None):
@@ -82,10 +95,11 @@ class ConfigPage(QWidget):
         raise NotImplementedError
 
 
-class ConfigDialog(QDialog):
+class ConfigDialog(QDialog, SizeMixin):
     """Spyder configuration ('Preferences') dialog box"""
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
+        SizeMixin.__init__(self)
         
         # Destroying the C++ object right after closing the dialog box,
         # otherwise it may be garbage-collected in another QThread
@@ -617,19 +631,30 @@ class MainConfigPage(GeneralConfigPage):
         return get_icon("genprefs.png")
     
     def setup_page(self):
+        newcb = self.create_checkbox
+
+        # --- Interface
         interface_group = QGroupBox(_("Interface"))
         styles = [str(txt) for txt in QStyleFactory.keys()]
         choices = zip(styles, [style.lower() for style in styles])
         style_combo = self.create_combobox(_('Qt windows style'), choices,
                                            'windows_style',
                                            default=self.main.default_style)
-        newcb = self.create_checkbox
+
+        single_instance_box = newcb(_("Use a single instance"),
+                                    'single_instance',
+                                    tip=_("Set this to open external<br> "
+                                          "Python files in an already running "
+                                          "instance (Requires a restart)"))
         vertdock_box = newcb(_("Vertical dockwidget title bars"),
                              'vertical_dockwidget_titlebars')
         verttabs_box = newcb(_("Vertical dockwidget tabs"),
                              'vertical_tabs')
         animated_box = newcb(_("Animated toolbars and dockwidgets"),
                              'animated_docks')
+        tear_off_box = newcb(_("Tear off menus"), 'tear_off_menus',
+                             tip=_("Set this to detach any<br> "
+                                   "menu from the main window"))
         margin_box = newcb(_("Custom dockwidget margin:"),
                            'use_custom_margin')
         margin_spin = self.create_spinbox("", "pixels", 'custom_margin',
@@ -640,20 +665,65 @@ class MainConfigPage(GeneralConfigPage):
         margins_layout = QHBoxLayout()
         margins_layout.addWidget(margin_box)
         margins_layout.addWidget(margin_spin)
+
+        # Decide if it's possible to activate or not singie instance mode
+        if os.name == 'nt':
+            pywin32_present = programs.is_module_installed('win32api')
+            if not pywin32_present:
+                self.set_option("single_instance", False)
+                single_instance_box.setEnabled(False)
+                tip = _("This feature requires the pywin32 module.\n"
+                        "It seems you don't have it installed.")
+                single_instance_box.setToolTip(tip)
+        elif sys.platform == "darwin" and 'Spyder.app' in __file__:
+            self.set_option("single_instance", True)
+            single_instance_box.setEnabled(False)
         
         interface_layout = QVBoxLayout()
         interface_layout.addWidget(style_combo)
+        interface_layout.addWidget(single_instance_box)
         interface_layout.addWidget(vertdock_box)
         interface_layout.addWidget(verttabs_box)
         interface_layout.addWidget(animated_box)
+        interface_layout.addWidget(tear_off_box)
         interface_layout.addLayout(margins_layout)
         interface_group.setLayout(interface_layout)
 
+        # --- Status bar
+        sbar_group = QGroupBox(_("Status bar"))
+        memory_box = newcb(_("Show memory usage every"), 'memory_usage/enable',
+                           tip=self.main.mem_status.toolTip())
+        memory_spin = self.create_spinbox("", " ms", 'memory_usage/timeout',
+                                          min_=100, max_=1000000, step=100)
+        self.connect(memory_box, SIGNAL("toggled(bool)"),
+                     memory_spin.setEnabled)
+        memory_spin.setEnabled(self.get_option('memory_usage/enable'))
+        memory_layout = QHBoxLayout()
+        memory_layout.addWidget(memory_box)
+        memory_layout.addWidget(memory_spin)
+        memory_layout.setEnabled(self.main.mem_status.is_supported())
+        cpu_box = newcb(_("Show CPU usage every"), 'cpu_usage/enable',
+                        tip=self.main.cpu_status.toolTip())
+        cpu_spin = self.create_spinbox("", " ms", 'cpu_usage/timeout',
+                                       min_=100, max_=1000000, step=100)
+        self.connect(cpu_box, SIGNAL("toggled(bool)"), cpu_spin.setEnabled)
+        cpu_spin.setEnabled(self.get_option('cpu_usage/enable'))
+        cpu_layout = QHBoxLayout()
+        cpu_layout.addWidget(cpu_box)
+        cpu_layout.addWidget(cpu_spin)
+        cpu_layout.setEnabled(self.main.cpu_status.is_supported())
+        
+        sbar_layout = QVBoxLayout()
+        sbar_layout.addLayout(memory_layout)
+        sbar_layout.addLayout(cpu_layout)
+        sbar_group.setLayout(sbar_layout)
+
+        # --- Debugging
         debug_group = QGroupBox(_("Debugging"))
-        popup_console_box = newcb(_("Pop up internal console when errors "
-                                    "were intercepted"),
+        popup_console_box = newcb(_("Pop up internal console when internal "
+                                    "errors appear"),
                                   'show_internal_console_if_traceback',
-                                  default=False)
+                                  default=True)
         
         debug_layout = QVBoxLayout()
         debug_layout.addWidget(popup_console_box)
@@ -661,6 +731,7 @@ class MainConfigPage(GeneralConfigPage):
         
         vlayout = QVBoxLayout()
         vlayout.addWidget(interface_group)
+        vlayout.addWidget(sbar_group)
         vlayout.addWidget(debug_group)
         vlayout.addStretch(1)
         self.setLayout(vlayout)
@@ -724,7 +795,7 @@ class ColorSchemeConfigPage(GeneralConfigPage):
                     cs_layout.addWidget(cb_bold, row+1, 2)
                     cs_layout.addWidget(cb_italic, row+1, 3)
             cs_group.setLayout(cs_layout)
-            if tabname in COLOR_SCHEME_NAMES:
+            if tabname in sh.COLOR_SCHEME_NAMES:
                 def_btn = self.create_button(_("Reset to default values"),
                                          lambda: self.reset_to_default(tabname))
                 tabs.addTab(self.create_tab(cs_group, def_btn), tabname)
