@@ -10,7 +10,8 @@ from spyderlib.qt.QtGui import (QVBoxLayout, QDialog, QWidget, QGroupBox,
                                 QLabel, QPushButton, QCheckBox, QLineEdit,
                                 QComboBox, QHBoxLayout, QDialogButtonBox,
                                 QStackedWidget, QGridLayout, QSizePolicy,
-                                QRadioButton, QMessageBox)
+                                QRadioButton, QMessageBox, QFrame,
+                                QButtonGroup)
 from spyderlib.qt.QtCore import SIGNAL, SLOT, Qt
 from spyderlib.qt.compat import getexistingdirectory
 
@@ -19,33 +20,59 @@ import os.path as osp
 
 # Local imports
 from spyderlib.baseconfig import _
-from spyderlib.config import get_icon, CONF
-from spyderlib.utils.qthelpers import get_std_icon
+from spyderlib.config import CONF
+from spyderlib.utils.qthelpers import get_icon, get_std_icon
+from spyderlib.plugins.configdialog import SizeMixin, GeneralConfigPage
+
+
+CURRENT_INTERPRETER = _("Execute in current Python or IPython interpreter")
+DEDICATED_INTERPRETER = _("Execute in a new dedicated Python interpreter")
+SYSTERM_INTERPRETER = _("Execute in an external System terminal")
+
+CURRENT_INTERPRETER_OPTION = 'default/interpreter/current'
+DEDICATED_INTERPRETER_OPTION = 'default/interpreter/dedicated'
+SYSTERM_INTERPRETER_OPTION = 'default/interpreter/systerm'
+
+WDIR_USE_SCRIPT_DIR_OPTION = 'default/wdir/use_script_directory'
+WDIR_USE_FIXED_DIR_OPTION = 'default/wdir/use_fixed_directory'
+WDIR_FIXED_DIR_OPTION = 'default/wdir/fixed_directory'
+
+ALWAYS_OPEN_FIRST_RUN = _("Always show %s on a first file run")
+ALWAYS_OPEN_FIRST_RUN_OPTION = 'open_on_firstrun'
 
 
 class RunConfiguration(object):
     """Run configuration"""
     def __init__(self, fname=None):
-        self.args = ''
-        self.args_enabled = False
+        self.args = None
+        self.args_enabled = None
         self.wdir = None
-        self.wdir_enabled = False
-        if fname is not None:
+        self.wdir_enabled = None
+        self.current = None
+        self.systerm = None
+        self.interact = None
+        self.python_args = None
+        self.python_args_enabled = None
+        self.set(CONF.get('run', 'defaultconfiguration', default={}))
+        if fname is not None and\
+           CONF.get('run', WDIR_USE_SCRIPT_DIR_OPTION, True):
             self.wdir = osp.dirname(fname)
             self.wdir_enabled = True
-        self.current = False
-        self.systerm = False
-        self.interact = False
-        self.python_args = ''
-        self.python_args_enabled = False
         
     def set(self, options):
         self.args = options.get('args', '')
         self.args_enabled = options.get('args/enabled', False)
-        self.wdir = options.get('workdir', os.getcwdu())
-        self.wdir_enabled = options.get('workdir/enabled', False)
-        self.current = options.get('current', False)
-        self.systerm = options.get('systerm', False)
+        if CONF.get('run', WDIR_USE_FIXED_DIR_OPTION, False):
+            default_wdir = CONF.get('run', WDIR_FIXED_DIR_OPTION, os.getcwdu())
+            self.wdir = options.get('workdir', default_wdir)
+            self.wdir_enabled = True
+        else:
+            self.wdir = options.get('workdir', os.getcwdu())
+            self.wdir_enabled = options.get('workdir/enabled', False)
+        self.current = options.get('current',
+                           CONF.get('run', CURRENT_INTERPRETER_OPTION, True))
+        self.systerm = options.get('systerm',
+                           CONF.get('run', SYSTERM_INTERPRETER_OPTION, False))
         self.interact = options.get('interact', False)
         self.python_args = options.get('python_args', '')
         self.python_args_enabled = options.get('python_args/enabled', False)
@@ -110,8 +137,16 @@ class RunConfigOptions(QWidget):
     """Run configuration options"""
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
+
+        self.current_radio = None
+        self.dedicated_radio = None
+        self.systerm_radio = None
+
         self.runconf = RunConfiguration()
         
+        firstrun_o = CONF.get('run', ALWAYS_OPEN_FIRST_RUN_OPTION, False)
+
+        # --- General settings ----
         common_group = QGroupBox(_("General settings"))
         common_layout = QGridLayout()
         common_group.setLayout(common_layout)
@@ -136,19 +171,18 @@ class RunConfigOptions(QWidget):
         wd_layout.addWidget(browse_btn)
         common_layout.addLayout(wd_layout, 1, 1)
         
-        radio_group = QGroupBox(_("Interpreter"))
-        radio_layout = QVBoxLayout()
-        radio_group.setLayout(radio_layout)
-        self.current_radio = QRadioButton(_("Execute in current Python "
-                                            "or IPython interpreter"))
-        radio_layout.addWidget(self.current_radio)
-        self.new_radio = QRadioButton(_("Execute in a new dedicated "
-                                        "Python interpreter"))
-        radio_layout.addWidget(self.new_radio)
-        self.systerm_radio = QRadioButton(_("Execute in an external "
-                                            "system terminal"))
-        radio_layout.addWidget(self.systerm_radio)
+        # --- Interpreter ---
+        interpreter_group = QGroupBox(_("Interpreter"))
+        interpreter_layout = QVBoxLayout()
+        interpreter_group.setLayout(interpreter_layout)
+        self.current_radio = QRadioButton(CURRENT_INTERPRETER)
+        interpreter_layout.addWidget(self.current_radio)
+        self.dedicated_radio = QRadioButton(DEDICATED_INTERPRETER)
+        interpreter_layout.addWidget(self.dedicated_radio)
+        self.systerm_radio = QRadioButton(SYSTERM_INTERPRETER)
+        interpreter_layout.addWidget(self.systerm_radio)
         
+        # --- Dedicated interpreter ---
         new_group = QGroupBox(_("Dedicated Python interpreter"))
         self.connect(self.current_radio, SIGNAL("toggled(bool)"),
                      new_group.setDisabled)
@@ -163,18 +197,28 @@ class RunConfigOptions(QWidget):
         self.connect(self.pclo_cb, SIGNAL("toggled(bool)"),
                      self.pclo_edit.setEnabled)
         self.pclo_edit.setEnabled(False)
+        self.pclo_edit.setToolTip(_("<b>-u</b> is added to the "
+                                    "other options you set here"))
         new_layout.addWidget(self.pclo_edit, 2, 1)
-        pclo_label = QLabel(_("The <b>-u</b> option is "
-                              "added to these commands"))
-        pclo_label.setWordWrap(True)
-        new_layout.addWidget(pclo_label, 3, 1)
         
         #TODO: Add option for "Post-mortem debugging"
+
+        # Checkbox to preserve the old behavior, i.e. always open the dialog
+        # on first run
+        hline = QFrame()
+        hline.setFrameShape(QFrame.HLine)
+        hline.setFrameShadow(QFrame.Sunken)
+        self.firstrun_cb = QCheckBox(ALWAYS_OPEN_FIRST_RUN % _("this dialog"))
+        self.connect(self.firstrun_cb, SIGNAL("clicked(bool)"),
+                     self.set_firstrun_o)
+        self.firstrun_cb.setChecked(firstrun_o)
         
         layout = QVBoxLayout()
+        layout.addWidget(interpreter_group)
         layout.addWidget(common_group)
-        layout.addWidget(radio_group)
         layout.addWidget(new_group)
+        layout.addWidget(hline)
+        layout.addWidget(self.firstrun_cb)
         self.setLayout(layout)
 
     def select_directory(self):
@@ -198,7 +242,7 @@ class RunConfigOptions(QWidget):
         elif self.runconf.systerm:
             self.systerm_radio.setChecked(True)
         else:
-            self.new_radio.setChecked(True)
+            self.dedicated_radio.setChecked(True)
         self.interact_cb.setChecked(self.runconf.interact)
         self.pclo_cb.setChecked(self.runconf.python_args_enabled)
         self.pclo_edit.setText(self.runconf.python_args)
@@ -224,22 +268,14 @@ class RunConfigOptions(QWidget):
                                  _("The following working directory is "
                                    "not valid:<br><b>%s</b>") % wdir)
             return False
+    
+    def set_firstrun_o(self):
+        CONF.set('run', ALWAYS_OPEN_FIRST_RUN_OPTION,
+                 self.firstrun_cb.isChecked())
 
 
-class SizeMixin(object):
-    """Mixin to keep widget size accessible
-    even when C++ object has been deleted"""
-    def __init__(self):
-        self.win_size = None
-
-    def resizeEvent(self, event):
-        """Reimplement Qt method"""
-        QDialog.resizeEvent(self, event)
-        self.win_size = self.size()
-
-
-class RunConfigOneDialog(QDialog, SizeMixin):
-    """Run configuration dialog box: single file version"""
+class BaseRunConfigDialog(QDialog, SizeMixin):
+    """Run configuration dialog box, base widget"""
     def __init__(self, parent=None):
         QDialog.__init__(self, parent)
         SizeMixin.__init__(self)
@@ -249,31 +285,56 @@ class RunConfigOneDialog(QDialog, SizeMixin):
         # (e.g. the editor's analysis thread in Spyder), thus leading to
         # a segmentation fault on UNIX or an application crash on Windows
         self.setAttribute(Qt.WA_DeleteOnClose)
-        
-        self.filename = None
-        
-        self.runconfigoptions = RunConfigOptions(self)
-        
-        bbox = QDialogButtonBox(QDialogButtonBox.Cancel)
-        bbox.addButton(_("Run"), QDialogButtonBox.AcceptRole)
+
+        self.setWindowIcon(get_icon("run.png"))
+        layout = QVBoxLayout()
+        self.setLayout(layout)
+    
+    def add_widgets(self, *widgets_or_spacings):
+        """Add widgets/spacing to dialog vertical layout"""
+        layout = self.layout()
+        for widget_or_spacing in widgets_or_spacings:
+            if isinstance(widget_or_spacing, int):
+                layout.addSpacing(widget_or_spacing)
+            else:
+                layout.addWidget(widget_or_spacing)
+    
+    def add_button_box(self, stdbtns):
+        """Create dialog button box and add it to the dialog layout"""
+        bbox = QDialogButtonBox(stdbtns)
+        run_btn = bbox.addButton(_("Run"), QDialogButtonBox.AcceptRole)
+        self.connect(run_btn, SIGNAL('clicked()'), self.run_btn_clicked)
         self.connect(bbox, SIGNAL("accepted()"), SLOT("accept()"))
         self.connect(bbox, SIGNAL("rejected()"), SLOT("reject()"))
-
         btnlayout = QHBoxLayout()
         btnlayout.addStretch(1)
         btnlayout.addWidget(bbox)
+        self.layout().addLayout(btnlayout)
         
-        layout = QVBoxLayout()
-        layout.addWidget(self.runconfigoptions)
-        layout.addLayout(btnlayout)
-        self.setLayout(layout)
-
-        self.setWindowIcon(get_icon("run.png"))
+    def run_btn_clicked(self):
+        """Run button was just clicked"""
+        pass
         
     def setup(self, fname):
+        """Setup Run Configuration dialog with filename *fname*"""
+        raise NotImplementedError
+
+
+class RunConfigOneDialog(BaseRunConfigDialog):
+    """Run configuration dialog box: single file version"""
+    def __init__(self, parent=None):
+        BaseRunConfigDialog.__init__(self, parent)
+        self.filename = None
+        self.runconfigoptions = None
+        
+    def setup(self, fname):
+        """Setup Run Configuration dialog with filename *fname*"""
         self.filename = fname
+        self.runconfigoptions = RunConfigOptions(self)
         self.runconfigoptions.set(RunConfiguration(fname).get())
-        self.setWindowTitle(_("Run %s") % osp.basename(fname))
+        self.add_widgets(self.runconfigoptions)
+        self.add_button_box(QDialogButtonBox.Cancel)
+        self.setWindowTitle(_("Run settings for %s") % osp.basename(fname))
             
     def accept(self):
         """Reimplement Qt method"""
@@ -290,20 +351,20 @@ class RunConfigOneDialog(QDialog, SizeMixin):
         return self.runconfigoptions.runconf
 
 
-class RunConfigDialog(QDialog, SizeMixin):
+class RunConfigDialog(BaseRunConfigDialog):
     """Run configuration dialog box: multiple file version"""
     def __init__(self, parent=None):
-        QDialog.__init__(self, parent)
-        SizeMixin.__init__(self)
-        
-        # Destroying the C++ object right after closing the dialog box,
-        # otherwise it may be garbage-collected in another QThread
-        # (e.g. the editor's analysis thread in Spyder), thus leading to
-        # a segmentation fault on UNIX or an application crash on Windows
-        self.setAttribute(Qt.WA_DeleteOnClose)
-        
+        BaseRunConfigDialog.__init__(self, parent)
         self.file_to_run = None
+        self.combo = None
+        self.stack = None
         
+    def run_btn_clicked(self):
+        """Run button was just clicked"""
+        self.file_to_run = unicode(self.combo.currentText())
+        
+    def setup(self, fname):
+        """Setup Run Configuration dialog with filename *fname*"""
         combo_label = QLabel(_("Select a run configuration:"))
         self.combo = QComboBox()
         self.combo.setMaxVisibleItems(20)
@@ -312,28 +373,6 @@ class RunConfigDialog(QDialog, SizeMixin):
         
         self.stack = QStackedWidget()
 
-        bbox = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
-        run_btn = bbox.addButton(_("Run"), QDialogButtonBox.AcceptRole)
-        self.connect(run_btn, SIGNAL('clicked()'), self.run_btn_clicked)
-        self.connect(bbox, SIGNAL("accepted()"), SLOT("accept()"))
-        self.connect(bbox, SIGNAL("rejected()"), SLOT("reject()"))
-
-        btnlayout = QHBoxLayout()
-        btnlayout.addStretch(1)
-        btnlayout.addWidget(bbox)
-        
-        layout = QVBoxLayout()
-        layout.addWidget(combo_label)
-        layout.addWidget(self.combo)
-        layout.addSpacing(10)
-        layout.addWidget(self.stack)
-        layout.addLayout(btnlayout)
-        self.setLayout(layout)
-
-        self.setWindowTitle(_("Run configurations"))
-        self.setWindowIcon(get_icon("run.png"))
-        
-    def setup(self, fname):
         configurations = _get_run_configurations()
         for index, (filename, options) in enumerate(configurations):
             if fname == filename:
@@ -352,6 +391,11 @@ class RunConfigDialog(QDialog, SizeMixin):
         self.connect(self.combo, SIGNAL("currentIndexChanged(int)"),
                      self.stack.setCurrentIndex)
         self.combo.setCurrentIndex(index)
+
+        self.add_widgets(combo_label, self.combo, 10, self.stack)
+        self.add_button_box(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+
+        self.setWindowTitle(_("Run Settings"))
         
     def accept(self):
         """Reimplement Qt method"""
@@ -359,12 +403,89 @@ class RunConfigDialog(QDialog, SizeMixin):
         for index in range(self.stack.count()):
             filename = unicode(self.combo.itemText(index))
             runconfigoptions = self.stack.widget(index)
-            if not runconfigoptions.is_valid():
+            if index == self.stack.currentIndex() and\
+               not runconfigoptions.is_valid():
                 return
             options = runconfigoptions.get()
             configurations.append( (filename, options) )
         _set_run_configurations(configurations)
         QDialog.accept(self)
+
+
+class RunConfigPage(GeneralConfigPage):
+    """Default Run Settings configuration page"""
+    CONF_SECTION = "run"
+    def get_name(self):
+        return _("Run")
+    
+    def get_icon(self):
+        return get_icon("run.png")
+    
+    def setup_page(self):
+        run_dlg = _("Run Settings")
+        run_menu = _("Run")
+        about_label = QLabel(_("The following are the default <i>%s</i>. "\
+                               "These options may be overriden using the "\
+                               "<b>%s</b> dialog box (see the <b>%s</b> menu)"\
+                               ) % (run_dlg, run_dlg, run_menu))
+        about_label.setWordWrap(True)
+
+        interpreter_group = QGroupBox(_("Interpreter"))
+        interpreter_bg = QButtonGroup(interpreter_group)
+        self.current_radio = self.create_radiobutton(CURRENT_INTERPRETER,
+                                CURRENT_INTERPRETER_OPTION, True,
+                                button_group=interpreter_bg)
+        self.dedicated_radio = self.create_radiobutton(DEDICATED_INTERPRETER,
+                                DEDICATED_INTERPRETER_OPTION, False,
+                                button_group=interpreter_bg)
+        self.systerm_radio = self.create_radiobutton(SYSTERM_INTERPRETER,
+                                SYSTERM_INTERPRETER_OPTION, False,
+                                button_group=interpreter_bg)
+
+        interpreter_layout = QVBoxLayout()
+        interpreter_group.setLayout(interpreter_layout)
+        interpreter_layout.addWidget(self.current_radio)
+        interpreter_layout.addWidget(self.dedicated_radio)
+        interpreter_layout.addWidget(self.systerm_radio)
         
-    def run_btn_clicked(self):
-        self.file_to_run = unicode(self.combo.currentText())
+        wdir_group = QGroupBox(_("Working directory"))
+        wdir_bg = QButtonGroup(wdir_group)
+        wdir_label = QLabel(_("Default working directory is:"))
+        wdir_label.setWordWrap(True)
+        dirname_radio = self.create_radiobutton(_("the script directory"),
+                                WDIR_USE_SCRIPT_DIR_OPTION, True,
+                                button_group=wdir_bg)
+        thisdir_radio = self.create_radiobutton(_("the following directory:"),
+                                WDIR_USE_FIXED_DIR_OPTION, False,
+                                button_group=wdir_bg)
+        thisdir_bd = self.create_browsedir("", WDIR_FIXED_DIR_OPTION,
+                                           os.getcwdu())
+        self.connect(thisdir_radio, SIGNAL("toggled(bool)"),
+                     thisdir_bd.setEnabled)
+        self.connect(dirname_radio, SIGNAL("toggled(bool)"),
+                     thisdir_bd.setDisabled)
+        thisdir_layout = QHBoxLayout()
+        thisdir_layout.addWidget(thisdir_radio)
+        thisdir_layout.addWidget(thisdir_bd)
+
+        wdir_layout = QVBoxLayout()
+        wdir_layout.addWidget(wdir_label)
+        wdir_layout.addWidget(dirname_radio)
+        wdir_layout.addLayout(thisdir_layout)
+        wdir_group.setLayout(wdir_layout)
+
+        firstrun_cb = self.create_checkbox(
+                            ALWAYS_OPEN_FIRST_RUN % _("Run Settings dialog"),
+                            ALWAYS_OPEN_FIRST_RUN_OPTION, False)
+        
+        vlayout = QVBoxLayout()
+        vlayout.addWidget(about_label)
+        vlayout.addSpacing(10)
+        vlayout.addWidget(interpreter_group)
+        vlayout.addWidget(wdir_group)
+        vlayout.addWidget(firstrun_cb)
+        vlayout.addStretch(1)
+        self.setLayout(vlayout)
+
+    def apply_settings(self, options):
+        pass
