@@ -11,21 +11,20 @@
 # pylint: disable=R0911
 # pylint: disable=R0201
 
-import re
-import sre_constants
-import string
 import os
+import re
+import string
 
 from spyderlib.qt.QtGui import (QTextCursor, QColor, QFont, QApplication,
                                 QTextEdit, QTextCharFormat, QToolTip,
-                                QTextDocument, QListWidget, QPlainTextEdit,
-                                QPalette, QMainWindow, QTextOption)
-from spyderlib.qt.QtCore import QPoint, SIGNAL, Qt, QRegExp, QEventLoop
+                                QListWidget, QPlainTextEdit, QPalette,
+                                QMainWindow, QTextOption, QMouseEvent)
+from spyderlib.qt.QtCore import QPoint, SIGNAL, Qt, QEventLoop, QEvent
 
 
 # Local imports
 from spyderlib.widgets.sourcecode.terminal import ANSIEscapeCodeHandler
-from spyderlib.utils import sourcecode
+from spyderlib.widgets.mixins import BaseEditMixin
 
 
 class CompletionWidget(QListWidget):
@@ -166,7 +165,7 @@ class CompletionWidget(QListWidget):
         self.hide()
 
 
-class TextEditBaseWidget(QPlainTextEdit):
+class TextEditBaseWidget(QPlainTextEdit, BaseEditMixin):
     """
     Text edit base widget
     """
@@ -174,6 +173,7 @@ class TextEditBaseWidget(QPlainTextEdit):
     
     def __init__(self, parent=None):
         QPlainTextEdit.__init__(self, parent)
+        BaseEditMixin.__init__(self)
         self.setAttribute(Qt.WA_DeleteOnClose)
         
         self.extra_selections_dict = {}
@@ -181,8 +181,6 @@ class TextEditBaseWidget(QPlainTextEdit):
         self.connect(self, SIGNAL('textChanged()'), self.changed)
         self.connect(self, SIGNAL('cursorPositionChanged()'),
                      self.cursor_position_changed)
-
-        self.eol_chars = None
         
         self.indent_chars = " "*4
         
@@ -385,441 +383,16 @@ class TextEditBaseWidget(QPlainTextEdit):
         self.setWordWrapMode(wrap_mode)
         
         
-    #------Positions, coordinates (cursor, EOF, ...)
-    def is_position_sup(self, pos1, pos2):
-        """Return True is pos1 > pos2"""
-        return pos1 > pos2
-        
-    def is_position_inf(self, pos1, pos2):
-        """Return True is pos1 < pos2"""
-        return pos1 < pos2
-        
-    def get_position(self, position):
-        cursor = self.textCursor()
-        if position == 'cursor':
-            pass
-        elif position == 'sol':
-            cursor.movePosition(QTextCursor.StartOfBlock)
-        elif position == 'eol':
-            cursor.movePosition(QTextCursor.EndOfBlock)
-        elif position == 'eof':
-            cursor.movePosition(QTextCursor.End)
-        elif position == 'sof':
-            cursor.movePosition(QTextCursor.Start)
-        else:
-            # Assuming that input argument was already a position
-            return position
-        return cursor.position()
-        
-    def get_coordinates(self, position):
-        position = self.get_position(position)
-        cursor = self.textCursor()
-        cursor.setPosition(position)
-        point = self.cursorRect(cursor).center()
-        return point.x(), point.y()
-    
-    def get_cursor_line_column(self):
-        """Return cursor (line, column) numbers"""
-        cursor = self.textCursor()
-        return cursor.blockNumber(), cursor.columnNumber()
-        
-    def get_cursor_line_number(self):
-        cursor = self.textCursor()
-        return cursor.blockNumber()+1
-
-    def set_cursor_position(self, position):
-        position = self.get_position(position)
-        cursor = self.textCursor()
-        cursor.setPosition(position)
-        self.setTextCursor(cursor)
-        self.ensureCursorVisible()
-        
-    def move_cursor(self, chars=0):
-        """Move cursor to left or right (unit: characters)"""
-        direction = QTextCursor.Right if chars > 0 else QTextCursor.Left
-        for _i in range(abs(chars)):
-            self.moveCursor(direction, QTextCursor.MoveAnchor)
-
-    def is_cursor_on_first_line(self):
-        """Return True if cursor is on the first line"""
-        cursor = self.textCursor()
-        cursor.movePosition(QTextCursor.StartOfBlock)
-        return cursor.atStart()
-
-    def is_cursor_on_last_line(self):
-        """Return True if cursor is on the last line"""
-        cursor = self.textCursor()
-        cursor.movePosition(QTextCursor.EndOfBlock)
-        return cursor.atEnd()
-
-    def is_cursor_at_end(self):
-        """Return True if cursor is at the end of the text"""
-        return self.textCursor().atEnd()
-
-    def is_cursor_before(self, position, char_offset=0):
-        """Return True if cursor is before *position*"""
-        position = self.get_position(position) + char_offset
-        cursor = self.textCursor()
-        cursor.movePosition(QTextCursor.End)
-        if position < cursor.position():
-            cursor.setPosition(position)
-            return self.textCursor() < cursor
-                
-    def __move_cursor_anchor(self, what, direction, move_mode):
-        assert what in ('character', 'word', 'line')
-        if what == 'character':
-            if direction == 'left':
-                self.moveCursor(QTextCursor.PreviousCharacter, move_mode)
-            elif direction == 'right':
-                self.moveCursor(QTextCursor.NextCharacter, move_mode)
-        elif what == 'word':
-            if direction == 'left':
-                self.moveCursor(QTextCursor.PreviousWord, move_mode)
-            elif direction == 'right':
-                self.moveCursor(QTextCursor.NextWord, move_mode)
-        elif what == 'line':
-            if direction == 'down':
-                self.moveCursor(QTextCursor.NextBlock, move_mode)
-            elif direction == 'up':
-                self.moveCursor(QTextCursor.PreviousBlock, move_mode)
-                
-    def move_cursor_to_next(self, what='word', direction='left'):
-        """
-        Move cursor to next *what* ('word' or 'character')
-        toward *direction* ('left' or 'right')
-        """
-        self.__move_cursor_anchor(what, direction, QTextCursor.MoveAnchor)
-    
-
-    #------Selection
-    def clear_selection(self):
-        """Clear current selection"""
-        cursor = self.textCursor()
-        cursor.clearSelection()
-        self.setTextCursor(cursor)
-
-    def extend_selection_to_next(self, what='word', direction='left'):
-        """
-        Extend selection to next *what* ('word' or 'character')
-        toward *direction* ('left' or 'right')
-        """
-        self.__move_cursor_anchor(what, direction, QTextCursor.KeepAnchor)
-                
-    def select_current_block(self):
-        """
-        Select block under cursor
-        Block = group of lines separated by either empty lines or commentaries
-        """
-        cursor = self.textCursor()
-        def _is_separator(cursor):
-            cursor0 = QTextCursor(cursor)
-            cursor0.select(QTextCursor.BlockUnderCursor)
-            text = unicode(cursor0.selectedText())
-            return len(text.strip()) == 0 or text.lstrip()[0] == '#'
-        cursor.movePosition(QTextCursor.StartOfBlock)
-        cur_pos = prev_pos = cursor.position()
-        while _is_separator(cursor):
-            # Moving to the next code block
-            cursor.movePosition(QTextCursor.NextBlock)
-            prev_pos = cur_pos
-            cur_pos = cursor.position()
-            if cur_pos == prev_pos:
-                return
-        while not _is_separator(cursor):
-            # Moving to the previous code block
-            cursor.movePosition(QTextCursor.PreviousBlock)
-            prev_pos = cur_pos
-            cur_pos = cursor.position()
-            if cur_pos == prev_pos:
-                if _is_separator(cursor):
-                    return
-                else:
-                    break
-        cursor.setPosition(prev_pos)
-        while not _is_separator(cursor):
-            # Moving to the next code block
-            cursor.movePosition(QTextCursor.NextBlock,
-                                QTextCursor.KeepAnchor)
-            cur_pos = cursor.position()
-            if cur_pos == prev_pos:
-                cursor.movePosition(QTextCursor.EndOfBlock,
-                                    QTextCursor.KeepAnchor)
-                break
-            prev_pos = cur_pos
-        self.setTextCursor(cursor)
-
-
-    #------EOL characters
-    def set_eol_chars(self, text):
-        """
-        Set widget end-of-line (EOL) characters from text (analyzes text)
-        """
-        if not isinstance(text, basestring): # testing for QString (PyQt API#1)
-            text = unicode(text)
-        eol_chars = sourcecode.get_eol_chars(text)
-        if eol_chars is not None and self.eol_chars is not None:
-            self.document().setModified(True)
-        self.eol_chars = eol_chars
-        
-    def get_line_separator(self):
-        """Return line separator based on current EOL mode"""
-        if self.eol_chars is not None:
-            return self.eol_chars
-        else:
-            return os.linesep
-
-    def get_text_with_eol(self):
-        """
-        Same as 'toPlainText', replace '\n' by correct end-of-line characters
-        """
-        utext = unicode(self.toPlainText())
-        lines = utext.splitlines()
-        linesep = self.get_line_separator()
-        txt = linesep.join(lines)
-        if utext.endswith('\n'):
-            txt += linesep
-        return txt
-            
-
-    #------Text selection
-    def has_selected_text(self):
-        """Returns True if some text is selected"""
-        return bool(unicode(self.textCursor().selectedText()))
-
-    def get_selected_text(self):
-        """
-        Return text selected by current text cursor, converted in unicode
-        
-        Replace the unicode line separator character \u2029 by 
-        the line separator characters returned by get_line_separator
-        """
-        return unicode(self.textCursor().selectedText()).replace(u"\u2029",
-                                                     self.get_line_separator())
-
+    #------Reimplementing Qt methods
     def copy(self):
         """
         Reimplement Qt method
         Copy text to clipboard with correct EOL chars
         """
         QApplication.clipboard().setText(self.get_selected_text())
-    
-    def remove_selected_text(self):
-        """Delete selected text"""
-        self.textCursor().removeSelectedText()
-        
-    def replace(self, text, pattern=None):
-        """Replace selected text by *text*
-        If *pattern* is not None, replacing selected text using regular
-        expression text substitution"""
-        cursor = self.textCursor()
-        cursor.beginEditBlock()
-        if pattern is not None:
-            seltxt = unicode(cursor.selectedText())
-        cursor.removeSelectedText()
-        if pattern is not None:
-            text = re.sub(unicode(pattern), unicode(text), unicode(seltxt))
-        cursor.insertText(text)
-        cursor.endEditBlock()
         
 
     #------Text: get, set, ...
-    def __select_text(self, position_from, position_to):
-        position_from = self.get_position(position_from)
-        position_to = self.get_position(position_to)
-        cursor = self.textCursor()
-        cursor.setPosition(position_from)
-        cursor.setPosition(position_to, QTextCursor.KeepAnchor)
-        return cursor
-
-    def get_text_line(self, line_nb):
-        """Return text line at line number *line_nb*"""
-        # Taking into account the case when a file ends in an empty line,
-        # since splitlines doesn't return that line as the last element
-        # TODO: Make this function more efficient
-        try:
-            return unicode(self.toPlainText()).splitlines()[line_nb]
-        except IndexError:
-            return self.get_line_separator()
-    
-    def get_text(self, position_from, position_to):
-        """
-        Return text between *position_from* and *position_to*
-        Positions may be positions or 'sol', 'eol', 'sof', 'eof' or 'cursor'
-        """
-        cursor = self.__select_text(position_from, position_to)
-        text = unicode(cursor.selectedText())
-        if text:
-            while text.endswith("\n"):
-                text = text[:-1]
-            while text.endswith(u"\u2029"):
-                text = text[:-1]
-        return text
-    
-    def get_character(self, position):
-        """Return character at *position*"""
-        position = self.get_position(position)
-        cursor = self.textCursor()
-        cursor.movePosition(QTextCursor.End)
-        if position < cursor.position():
-            cursor.setPosition(position)
-            cursor.movePosition(QTextCursor.Right,
-                                QTextCursor.KeepAnchor)
-            return unicode(cursor.selectedText())
-        else:
-            return ''
-    
-    def insert_text(self, text):
-        """Insert text at cursor position"""
-        if not self.isReadOnly():
-            self.textCursor().insertText(text)
-    
-    def replace_text(self, position_from, position_to, text):
-        cursor = self.__select_text(position_from, position_to)
-        cursor.removeSelectedText()
-        cursor.insertText(text)
-        
-    def remove_text(self, position_from, position_to):
-        cursor = self.__select_text(position_from, position_to)
-        cursor.removeSelectedText()
-        
-    def find_multiline_pattern(self, regexp, cursor, findflag):
-        """Reimplement QTextDocument's find method
-        
-        Add support for *multiline* regular expressions"""
-        pattern = unicode(regexp.pattern())
-        text = unicode(self.toPlainText())
-        try:
-            regobj = re.compile(pattern)
-        except sre_constants.error:
-            return
-        if findflag & QTextDocument.FindBackward:
-            # Find backward
-            offset = min([cursor.selectionEnd(), cursor.selectionStart()])
-            text = text[:offset]
-            matches = [_m for _m in regobj.finditer(text, 0, offset)]
-            if matches:
-                match = matches[-1]
-            else:
-                return
-        else:
-            # Find forward
-            offset = max([cursor.selectionEnd(), cursor.selectionStart()])
-            match = regobj.search(text, offset)
-        if match:
-            pos1, pos2 = match.span()
-            fcursor = self.textCursor()
-            fcursor.setPosition(pos1)
-            fcursor.setPosition(pos2, QTextCursor.KeepAnchor)
-            return fcursor
-
-    def find_text(self, text, changed=True, forward=True, case=False,
-                  words=False, regexp=False):
-        """Find text"""
-        cursor = self.textCursor()
-        findflag = QTextDocument.FindFlag()
-        if not forward:
-            findflag = findflag | QTextDocument.FindBackward
-        moves = [QTextCursor.NoMove]
-        if forward:
-            moves += [QTextCursor.NextWord, QTextCursor.Start]
-            if changed:
-                if unicode(cursor.selectedText()):
-                    new_position = min([cursor.selectionStart(),
-                                        cursor.selectionEnd()])
-                    cursor.setPosition(new_position)
-                else:
-                    cursor.movePosition(QTextCursor.PreviousWord)
-        else:
-            moves += [QTextCursor.End]
-        if not regexp:
-            text = re.escape(unicode(text))
-        pattern = QRegExp(r"\b%s\b" % text if words else text,
-                          Qt.CaseSensitive if case else Qt.CaseInsensitive,
-                          QRegExp.RegExp2)
-        for move in moves:
-            cursor.movePosition(move)
-            if regexp and '\\n' in text:
-                # Multiline regular expression
-                found_cursor = self.find_multiline_pattern(pattern, cursor,
-                                                           findflag)
-            else:
-                # Single line find: using the QTextDocument's find function,
-                # probably much more efficient than ours
-                found_cursor = self.document().find(pattern, cursor, findflag)
-            if found_cursor is not None and not found_cursor.isNull():
-                self.setTextCursor(found_cursor)
-                return True
-        return False
-    
-    def get_current_word(self):
-        """Return current word, i.e. word at cursor position"""
-        cursor = self.textCursor()
-
-        if cursor.hasSelection():
-            # Removes the selection and moves the cursor to the left side 
-            # of the selection: this is required to be able to properly 
-            # select the whole word under cursor (otherwise, the same word is 
-            # not selected when the cursor is at the right side of it):
-            cursor.setPosition(min([cursor.selectionStart(),
-                                    cursor.selectionEnd()]))
-        else:
-            # Checks if the first character to the right is a white space
-            # and if not, moves the cursor one word to the left (otherwise,
-            # if the character to the left do not match the "word regexp" 
-            # (see below), the word to the left of the cursor won't be 
-            # selected), but only if the first character to the left is not a
-            # white space too.
-            def is_space(move):
-                curs = self.textCursor()
-                curs.movePosition(move, QTextCursor.KeepAnchor)
-                return not unicode(curs.selectedText()).strip()
-            if is_space(QTextCursor.NextCharacter):
-                if is_space(QTextCursor.PreviousCharacter):
-                    return
-                cursor.movePosition(QTextCursor.WordLeft)
-
-        cursor.select(QTextCursor.WordUnderCursor)
-        text = unicode(cursor.selectedText())
-        match = re.findall(r'([a-zA-Z\_]+[0-9a-zA-Z\_]*)', text)
-        if match:
-            return match[0]
-    
-    def get_current_line(self):
-        """Return current line's text"""
-        cursor = self.textCursor()
-        cursor.select(QTextCursor.BlockUnderCursor)
-        return unicode(cursor.selectedText())
-    
-    def get_line_number_at(self, coordinates):
-        """Return line number at *coordinates* (QPoint)"""
-        cursor = self.cursorForPosition(coordinates)
-        return cursor.blockNumber()-1
-    
-    def get_line_at(self, coordinates):
-        """Return line at *coordinates* (QPoint)"""
-        cursor = self.cursorForPosition(coordinates)
-        cursor.select(QTextCursor.BlockUnderCursor)
-        return unicode(cursor.selectedText()).replace(u'\u2029', '')
-    
-    def get_word_at(self, coordinates):
-        """Return word at *coordinates* (QPoint)"""
-        cursor = self.cursorForPosition(coordinates)
-        cursor.select(QTextCursor.WordUnderCursor)
-        return unicode(cursor.selectedText())
-    
-    def get_block_indentation(self, block_nb):
-        """Return line indentation (character number)"""
-        text = unicode(self.document().findBlockByNumber(block_nb).text())
-        return len(text)-len(text.lstrip())
-    
-    def get_selection_bounds(self):
-        """Return selection bounds (block numbers)"""
-        cursor = self.textCursor()
-        start, end = cursor.selectionStart(), cursor.selectionEnd()
-        block_start = self.document().findBlock(start)
-        block_end = self.document().findBlock(end)
-        return sorted([block_start.blockNumber(), block_end.blockNumber()])
-        
     def get_executable_text(self):
         """Return selected text or current line as an processed text,
         to be executable in a Python/IPython interpreter"""
@@ -839,31 +412,64 @@ class TextEditBaseWidget(QPlainTextEdit):
             # Multiline selection -> eventually fixing indentation
             original_indent = _indent(self.get_text_line(line_from))
             text = (" "*(original_indent-_indent(lines[0])))+text
-        
-        # If there is a common indent to all lines, remove it
+            
+        # If there is a common indent to all lines, find it.
+        # Moving from bottom line to top line ensures that blank
+        # lines inherit the indent of the line *below* it,
+        # which is the desired behavior.
         min_indent = 999
-        for line in text.split(ls):
+        current_indent = 0
+        lines = text.split(ls)
+        for i in xrange(len(lines)-1, -1, -1):
+            line = lines[i]
             if line.strip():
-                min_indent = min(_indent(line), min_indent)
+                current_indent = _indent(line)
+                min_indent = min(current_indent, min_indent)
+            else:
+                lines[i] = ' ' * current_indent
         if min_indent:
-            text = ls.join([line[min_indent:] for line in text.split(ls)])
-        
-        # Add an EOL character if a block stars with various Python
-        # reserved words, so that it gets evaluated automatically
-        # by the console
-        first_line = lines[0].lstrip()
-        last_line = self.get_text_line(line_to).strip()
-        words = ['def', 'for', 'if', 'while', 'try', 'with', 'class']
-        if any([first_line.startswith(w) for w in words]):
-            text += ls
-            if last_line != '':
-                text += ls
+            lines = [line[min_indent:] for line in lines]
 
+        # Remove any leading whitespace or comment lines
+        # since they confuse the reserved word detector that follows below
+        first_line = lines[0].lstrip()
+        while first_line == '' or first_line[0] == '#':
+            lines.pop(0)
+            first_line = lines[0].lstrip()
+        
+        # Add an EOL character after indentation blocks that start with some 
+        # Python reserved words, so that it gets evaluated automatically
+        # by the console
+        varname = re.compile('[a-zA-Z0-9_]*') # matches valid variable names
+        maybe = False
+        nextexcept = ()
+        for n, line in enumerate(lines):
+            if not _indent(line):
+                word = varname.match(line).group()
+                if maybe and word not in nextexcept:
+                    lines[n-1] += ls
+                    maybe = False
+                if word:
+                    if word in ('def', 'for', 'while', 'with', 'class'):
+                        maybe = True
+                        nextexcept = ()
+                    elif word == 'if':
+                        maybe = True
+                        nextexcept = ('elif', 'else')
+                    elif word == 'try':
+                        maybe = True
+                        nextexcept = ('except', 'finally')
+        if maybe:
+            if lines[-1].strip() == '':
+                lines[-1] += ls
+            else:
+                lines.append(ls)
+        
         if no_selection:
             self.setFocus()
             self.clear_selection()
         
-        return text
+        return ls.join(lines)
     
     def get_line_count(self):
         """Return document total line number"""
@@ -1041,9 +647,7 @@ class TextEditBaseWidget(QPlainTextEdit):
         cx, cy = self.get_coordinates('cursor')
         if at_line is not None:
             cx = 5
-            cursor = self.textCursor()
-            block = self.document().findBlockByNumber(at_line-1)
-            cursor.setPosition(block.position())
+            cursor = QTextCursor(self.document().findBlockByNumber(at_line-1))
             cy = self.cursorRect(cursor).top()
         point = self.mapToGlobal(QPoint(cx, cy))
         point.setX(point.x()+self.get_linenumberarea_width())
@@ -1161,7 +765,19 @@ class TextEditBaseWidget(QPlainTextEdit):
         pass
 
                 
-    #----Focus
+    #----Qt Events
+    def mousePressEvent(self, event):
+        """Reimplement Qt method"""
+        if os.name != 'posix' and event.button() == Qt.MidButton:
+            self.setFocus()
+            event = QMouseEvent(QEvent.MouseButtonPress, event.pos(),
+                                Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)
+            QPlainTextEdit.mousePressEvent(self, event)
+            QPlainTextEdit.mouseReleaseEvent(self, event)
+            self.paste()
+        else:
+            QPlainTextEdit.mousePressEvent(self, event)
+
     def focusInEvent(self, event):
         """Reimplemented to handle focus"""
         self.emit(SIGNAL("focus_changed()"))
