@@ -28,9 +28,8 @@ import re
 
 # Local imports
 from spyderlib.utils import programs
-from spyderlib.utils.qthelpers import create_toolbutton
+from spyderlib.utils.qthelpers import get_icon, create_toolbutton
 from spyderlib.baseconfig import get_conf_path, get_translation
-from spyderlib.config import get_icon
 from spyderlib.widgets.onecolumntree import OneColumnTree
 from spyderlib.widgets.texteditor import TextEditor
 from spyderlib.widgets.comboboxes import (PythonModulesComboBox,
@@ -50,12 +49,17 @@ class ResultsTree(OneColumnTree):
         self.data = None
         self.set_title('')
         
-    def activated(self):
-        data = self.data.get(self.currentItem())
+    def activated(self, item):
+        """Double-click event"""
+        data = self.data.get(item)
         if data is not None:
             fname, lineno = data
             self.parent().emit(SIGNAL("edit_goto(QString,int,QString)"),
                                fname, lineno, '')
+
+    def clicked(self, item):
+        """Click event"""
+        self.activated(item)
         
     def clear_results(self):
         self.clear()
@@ -95,7 +99,7 @@ class ResultsTree(OneColumnTree):
                     i_base = module.find(basename)
                     module = module[i_base:]
                 dirname = osp.dirname(self.filename)
-                if module.startswith('.'):
+                if module.startswith('.') or module == basename:
                     modname = osp.join(dirname, module)
                 else:
                     modname = osp.join(dirname, *module.split('.'))
@@ -265,12 +269,12 @@ class PylintWidget(QWidget):
         index, _data = self.get_data(filename)
         if index is not None:
             self.rdata.pop(index)
-        self.rdata.append( (filename, data) )
+        self.rdata.insert(0, (filename, data))
         self.save()
         
     def save(self):
         while len(self.rdata) > self.max_entries:
-            self.rdata.pop(0)
+            self.rdata.pop(-1)
         cPickle.dump([self.VERSION]+self.rdata, file(self.DATAPATH, 'w'))
         
     def show_log(self):
